import json, os

ctpath = os.path.join(os.path.dirname(__file__), "component_types.json")

with open(ctpath, "r") as f:
    raw_data = json.load(f)

class CustomFieldType:
    def __init__(self, name, data, proproot):
        self.name = name
        self.type = data["type"]
        self.subtype = data.get("subtype")
        self.cprop = proproot + self.type + "_" + name
        if self.type == "float":
            self.default = 0.0
        elif self.type == "float2":
            self.default = (0.0, 0.0)
        elif self.type == "float3":
            self.default = (0.0, 0.0, 0.0)
        elif self.type == "float4":
            self.default = (0.0, 0.0, 0.0, 0.0)
        elif self.type == "int":
            self.default = 0
        elif self.type == "int2":
            self.default = (0, 0)
        elif self.type == "int3":
            self.default = (0, 0, 0)
        elif self.type == "int4":
            self.default = (0, 0, 0, 0)
        elif self.type == "entity":
            self.default = None
        elif self.type == "bool":
            self.default = False
        elif self.type == "bool2":
            self.default = (False, False)
        elif self.type == "bool3":
            self.default = (False, False, False)
        elif self.type == "bool4":
            self.default = (False, False, False, False)

        self.ui_param = None
        self.icon = data.get("icon")
        if self.subtype == "COLOR":
            self.ui_param = dict(min=0, max=1)
            if self.type == "float":
                self.default = 1.0
            elif self.type == "float2":
                self.default = (1.0, 1.0)
            elif self.type == "float3":
                self.default = (1.0, 1.0, 1.0)
            elif self.type == "float4":
                self.default = (1.0, 1.0, 1.0, 1.0)
        else:
            minv, maxv = data.get("min", None), data.get("max", None)
            self.ui_param = dict()
            if minv != None: self.ui_param["min"] = minv
            if maxv != None: self.ui_param["max"] = maxv

class CustomComponentType:
    def __init__(self, name, data):
        self.name = name
        self.fields = []
        self.cprop_prefix = "PC_" + name + "_"
        self.icon = data.get("icon")
        if "fields" in data.keys():
            for f_name in data["fields"].keys():
                self.fields.append(CustomFieldType(f_name, data["fields"][f_name], self.cprop_prefix))

component_types = [CustomComponentType(comp_name, raw_data["components"][comp_name]) for comp_name in raw_data["components"].keys()]